using System.Linq;
using NHibernate.Linq;
using NUnit.Framework;

namespace NHibernate.Test.NHSpecificTest.NH2856
{
    [TestFixture]
    public class Fixture : BugTestCase
    {
        protected override void OnSetUp()
        {
            base.OnSetUp();

            using (var session = OpenSession())
            {
                using (var tran = session.BeginTransaction())
                {
                    Person person = new Person();
                    person.Name = "John Doe";
                    session.Save(person);

                    Address address = new Address();
                    address.Name = "Home";
                    session.Save(address);

                    person.Address = address;
                    session.Flush();

                    tran.Commit();
                }
            }
        }

        protected override void OnTearDown()
        {
            base.OnTearDown();

            using (var session = OpenSession())
            {
                using (var tran = session.BeginTransaction())
                {
                    session.Delete("from Person");
                    session.Delete("from Address");
                    tran.Commit();
                }
            }
        }

        //tests NH2673
        [Test]
        public void CacheQueryWithFetch()
        {
            using(var session = OpenSession())
            {
                var query = session.Query<Person>()
                    .Fetch(p => p.Address)
                    .Cacheable();

                var results = query.ToList();
            }
        }

        [Test]
        public void UseCachedQueryWithFetch()
        {
            using (var session = OpenSession())
            {
                var query = session.Query<Person>()
                    .Fetch(p => p.Address)
                    .Cacheable();

                var results = query.ToList();

                using(var spy = new SqlLogSpy())
                {
                    var cached = query.ToList();
    
                    Assert.IsEmpty(spy.GetWholeLog());
                    Assert.AreEqual(results.Count, cached.Count);
                }

                
            }
        }
    }
}
